/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

    Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Tray.h"

#include "GhostTray.h"
#include "GhostButton.h"

#include "shlwapi.h"

//No need for seperate Owners, since ONLY 1 Tray is possible
//xTaskbar and xLabel are different, because there can be more than one
extern HWND messageHandler;

extern int bitsperpixel;

int iRealMonitors = GetSystemMetrics(SM_CMONITORS);

extern int iScreenWidth;
extern int iScreenHeight;

//Alpha Transparency
typedef BOOL (WINAPI *PULW)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
static PULW pUpdateLayeredWindow = NULL;

#define TIMER_MOUSETRACK	1
#define TIMER_ALPHAFADEIN	2
#define TIMER_ALPHAFADEOUT	3
#define TIMER_CLEAN			4
#define TIMER_INFOTIPTIMEOUT 5
#define TIMER_SECURESTART	 6
#define TIMER_TRAYADD		 7
#define TIMER_TRAYREMOVE	 8

#define TIMER_ALPHAFADEUPBANG 12
#define TIMER_ALPHAFADEDOWNBANG 13

#define INFOTIMER_AUTOHIDE 100

UINT lsMessages[] = {	LM_SYSTRAY,
						LM_REFRESH,
						0	};

#define INFOTIP_MAXIMUM_WIDTH 300
#define INFOTIP_MAXIMUM_TIMEOUT 15000
#define INFOTIP_MINIMUM_TIMEOUT 7500

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Tray::Tray(const string &name) :
	actualx(0),
	actualy(0),
	actualwidth(0),
	actualheight(0),
	
	hWnd(NULL),
	hInstance(NULL),
	box(NULL),

	bTrueTransparency(false),

	//-----------------------------------------------
	// Temporary Help Pointers
	//-----------------------------------------------

	pMoveCapture(NULL),
	pCapture(NULL),
	pChangeState(NULL),
	pAddedTrayIcon(NULL),
	pDontPaintAddedTrayIcon(NULL),

	hWndToolTip(NULL),
	pTrayTooltip(NULL),

	hWndBehindWindow(NULL),

	oldbackupBMP(NULL),
	backupDC(NULL),
	backupBitmap(NULL),
	bSkipBackup(false),

	bBlocked(false),

	bParentDestroyNotificationReceived(false),

	bRemovedTrayIcon(false),

	iInternalAddCount(0),

	hWndInfoTip(NULL),
	bInfoTipsEnabled(false),

	iStartWithPosition(0),
	iMaxIconSize(16),

	iAutoSizeSteps(0),
	iAutoSizeDelay(0),
	
	iBackupExportedButtonCount(0),

	iLines_RowsAdded(0),

	bBorderDrag(false),

	bAutoHidden(false),

	iUpdateAlpha(0),

	bAlphaFade(false),

	iAlphaFadeStep(2),
	iAlphaFadeDelay(10),
	
	bAlphaInitialized(false),
	bLayeredFlag(false),

	bMouseInside(false),

	bSkipButton(false),

	bVisible(false),

	iVisibleIcons(0),
	bHiddenVisible(false),

	bInCriticalProcess(false),

	pTrayTexture(NULL),

	pNormalIcon(NULL),
	pHoverIcon(NULL),
	pHiddenIcon(NULL)
{
	strName = name;
	
	UpdateConfig.x = 0;
	UpdateConfig.y = 0;
	UpdateConfig.button = NULL;
}

Tray::~Tray()
{
	HideList.clear();
	NoDragList.clear();
	SortList.clear();

	if (pTrayTooltip)
	{
		Destroy_xTooltipClass(pTrayTooltip);
		pTrayTooltip = NULL;
	}

	if ( bShowInfoTips )
		DestroyInfoTip();

	// release buttons
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
	{	
		delete (*tbit);
		*tbit = NULL;
	}
	listTrayButtons.clear();

	// unregister messages
	SendMessage( GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );

	if(IsWindow(hWnd))
	{	
		// shutdown timer
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_CLEAN);
		KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
		KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
		KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);

		DestroyWindow(hWnd);
	}

	if ( backupBitmap )
	{
		backupBitmap = (HBITMAP) SelectObject(backupDC, oldbackupBMP);
		DeleteObject(backupBitmap);
	}
	if ( backupDC )
		DeleteDC(backupDC);
	if ( oldbackupBMP )
		DeleteObject(oldbackupBMP);

	backupDC = NULL;
	backupBitmap = NULL;
	oldbackupBMP = NULL;

	for(GhostTrayListIterator gtlit = listGhostTrays.begin(); gtlit != listGhostTrays.end(); gtlit++)
	{
		delete *gtlit;
		*gtlit = NULL;
	}
	listGhostTrays.clear();

	for(GhostButtonListIterator glit = listGhostButtons.begin(); glit != listGhostButtons.end(); glit++)
	{
		delete *glit;
		*glit = NULL;
	}
	listGhostButtons.clear();

	if (pTrayTexture)
		Destroy_xTextureClass(pTrayTexture);

	if (pNormalIcon)
		Destroy_xIconClass(pNormalIcon);
	if (pHoverIcon)
		Destroy_xIconClass(pHoverIcon);
	if (pHiddenIcon)
		Destroy_xIconClass(pHiddenIcon);
}

void Tray::Load(HINSTANCE hInstance, HWND setbox)
{
	this->hInstance = hInstance;

	if (IsWindow(setbox))
	{
		box = setbox;
	}
	else
	{
		box = NULL;
	}
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	hWnd = CreateWindowEx( box ? 0 : WS_EX_TOOLWINDOW,
							"xTray",
							strName.c_str(),
							(box ? WS_CHILD : WS_POPUP),
							actualx, actualy,		
							actualwidth, actualheight,
							(box ? box : messageHandler), 
							0,
							hInstance,
							this);
		
	if (hWnd)
	{
		PrefixedLSSetVariable(strName, "CurrentHWND", IntToString( (int) hWnd ));

		SendMessage( GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );
		
		SetWindowLong(hWnd, GWL_USERDATA, MAGIC_DWORD);

		Reconfigure();
	
		if ( !bHideIfEmpty || !bAutoHidden )
		{
			if ( bStartHidden )
				Hide();
			else
				Show();

			SetTimer(hWnd, TIMER_SECURESTART, 10000, NULL);

			bBlocked = true;
			
			SendMessage(GetLitestepWnd(), LM_SYSTRAYREADY, NULL, NULL);
	
			bBlocked = false;
		}
		
		bBlocked = true;

		if ( bAutoSize )
			AutoSizeFunction();
		
		CreateLayout();

		bBlocked = false;

		iVisibleIcons = 0;
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{
			if ( !(*tbit)->bxTrayHideChecked )
			{
				if ( IsOnTrayList((*tbit)->hWndOwner, HideList, false) )
					(*tbit)->bUserHidden = true;

				(*tbit)->bxTrayHideChecked = true;
			}

			if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
				iVisibleIcons++;
		}

		if ( iWrapCount != -1 )
		{
			iLines_RowsAdded = 0;

			int count = iVisibleIcons/iWrapCount;
			int rest = iVisibleIcons%iWrapCount;
				
			int todo = count-iLines_Rows;
			
			if ( count != 0 && rest != 0 )
				todo++;

			while ( todo > 0 && iLines_Rows+iLines_RowsAdded+1 <= iMaxLines_Rows )
			{
				iLines_RowsAdded++;
				PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));

				if ( !trayLineAdd.empty() ) 
					LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

				todo--;
			}
		
			PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));
		}

		if ( !trayAdd.empty() )
		{
			int todo = iVisibleIcons;
			while ( todo > 0 )
			{
				PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons-todo+1 ));
				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		CreateLayout();
	}
	else
	{
		delete this;
	}
}

void Tray::Create_xPaintClassSupport()
{
	if (!pTrayTexture)
		pTrayTexture = Create_xTextureClass();

	if (!pNormalIcon)
		pNormalIcon = Create_xIconClass();
	if (!pHoverIcon)
		pHoverIcon = Create_xIconClass();
	if (!pHiddenIcon)
		pHiddenIcon = Create_xIconClass();

	if ( IsOS(OS_2KXP) && Getx2RCBool(strName.c_str(), "AlphaMap") )
	{
		pTrayTexture->configure(strName.c_str(), "", true);

		PaintIcon *defaulticon = Create_xIconClass();
		defaulticon->configure(strName.c_str(), "", false, true);
		
		pNormalIcon->configure(strName.c_str(), "Normal", false, true, defaulticon);
		if ( !pHoverIcon->configure(strName.c_str(), "Hover", false, true, pNormalIcon) )
		{
			Destroy_xIconClass(pHoverIcon);
			pHoverIcon = NULL;
		}
		if ( !pHiddenIcon->configure(strName.c_str(), "Hidden", false, true, pNormalIcon) )
		{
			Destroy_xIconClass(pHiddenIcon);
			pHiddenIcon = NULL;
		}
		
		//CleanUp DefaultIcon
		Destroy_xIconClass(defaulticon);
	}
	else
	{
		pTrayTexture->configure(strName.c_str(), "", false);

		PaintIcon *defaulticon = Create_xIconClass();
		defaulticon->configure(strName.c_str(), "", false, false);
		
		pNormalIcon->configure(strName.c_str(), "Normal", false, false, defaulticon);
		if ( !pHoverIcon->configure(strName.c_str(), "Hover", false, false, pNormalIcon) )
		{
			Destroy_xIconClass(pHoverIcon);
			pHoverIcon = NULL;
		}
		if ( !pHiddenIcon->configure(strName.c_str(), "Hidden", false, false, pNormalIcon) )
		{
			Destroy_xIconClass(pHiddenIcon);
			pHiddenIcon = NULL;
		}
		
		//CleanUp DefaultIcon
		Destroy_xIconClass(defaulticon);
	}
}

void Tray::Reconfigure(bool refresh)
{
	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	SetWindowRgn(hWnd, NULL, true);

	const NameValuePair directionValues[] = {
			{ ".right", 1 },
			{ ".left", 2 },
			{ ".bottom", 3 },
			{ ".top", 4 },
			{ "right", 1 },
			{ "left", 2 },
			{ "bottom", 3 },
			{ "top", 4 },
			{ 0, 0 }
			};

	const NameValuePair wrapdirectionValues[] = {
			{ ".bottom", 1 },
			{ ".top", 2 },
			{ ".right", 3 },
			{ ".left", 4 },
			{ "bottom", 1 },
			{ "top", 2 },
			{ "right", 3 },
			{ "left", 4 },
			{ 0, 0 }
			};

	const NameValuePair layoutValues[] = {
			{ ".horizontal", 0 },
			{ ".vertical", 1 },
			{ "horizontal", 0 },
			{ "vertical", 1 },
			{ 0, 0 }
			};
	
	HideList.clear();
	NoDragList.clear();
	SortList.clear();

	//Create/Refresh all Painting Stuff
	Create_xPaintClassSupport();

	GetxRCList("*xTrayHide", HideList);
	GetxRCList("*xTrayNoDragNDrop", NoDragList);
	GetxRCList("*xTrayPosition", SortList, true);

	bNoDragnDrop = Getx2RCBool(strName.c_str(), "DisableDragnDrop");

	bBorderDrag = Getx2RCBool(strName.c_str(), "Moveable");

	bHideIfEmpty = Getx2RCBool(strName.c_str(), "HideIfEmpty");

	iCleanUpInterval = Getx2RCInt(strName.c_str(), "CleanUpInterval", 10000, 1000, 3600000);

	//Events/Actions
	trayLButtonUp = iGetx2RCLine(strName, "OnLeftClickUp", "");
	trayLButtonDown = iGetx2RCLine(strName, "OnLeftClickDown", "");
	trayMButtonUp = iGetx2RCLine(strName, "OnMiddleClickUp", "");
	trayMButtonDown = iGetx2RCLine(strName, "OnMiddleClickDown", "");
	trayRButtonUp = iGetx2RCLine(strName, "OnRightClickUp", "");
	trayRButtonDown = iGetx2RCLine(strName, "OnRightClickDown", "");

	bFireClicksAlways = Getx2RCBool(strName.c_str(), "AlwaysFireClickEvents");

	trayMWheelDown = iGetx2RCLine(strName, "OnWheelDown", "");
	trayMWheelUp = iGetx2RCLine(strName, "OnWheelUp", "");
	trayEnter = iGetx2RCLine(strName, "OnEnter", "");
	trayLeave = iGetx2RCLine(strName, "OnLeave", "");
	trayMove = iGetx2RCLine(strName, "OnMove", "");
	trayResize = iGetx2RCLine(strName, "OnResize", "");
	trayAdd = iGetx2RCLine(strName, "OnAdd", "");
	trayRemove = iGetx2RCLine(strName, "OnRemove", "");

	trayShow = iGetx2RCLine(strName, "OnShow", "");
	trayHide = iGetx2RCLine(strName, "OnHide", "");

	trayLineAdd = iGetx2RCLine(strName, "OnLineAdd", "");
	trayLineRemove = iGetx2RCLine(strName, "OnLineRemove", "");

	trayOnIconHide = iGetx2RCLine(strName, "OnIconHide", "");
	trayOnIconUnHide = iGetx2RCLine(strName, "OnIconUnHide", "");

	bStartHidden = Getx2RCBool(strName.c_str(), "StartHidden");

	bLoopingTrayIcons = Getx2RCBool(strName.c_str(), "LoopIcons");
	
	bFocusOnEnter = Getx2RCBool(strName.c_str(), "FocusOnEnter");

	string borders = iGetx2RCLine(strName, "Borders", "");
	
	if ( !borders.empty() )
	{
		numTokens = SplitString(borders, one, two, three, four, five, six, seven);
	
		if(numTokens == 1)
		{
			iLeftBorder = atoi(one.c_str());
			iRightBorder = iLeftBorder;
			iTopBorder = iLeftBorder;
			iBottomBorder = iLeftBorder;
		}
		else if (numTokens == 2)
		{
			iLeftBorder = atoi(one.c_str());
			iRightBorder = iLeftBorder;
			iTopBorder = atoi(two.c_str());
			iBottomBorder = iTopBorder;
		}
		else if (numTokens == 4)
		{
			iLeftBorder = atoi(one.c_str());
			iRightBorder = atoi(two.c_str());
			iTopBorder = atoi(three.c_str());
			iBottomBorder = atoi(four.c_str());
		}

		if ( iLeftBorder < 0 )
			iLeftBorder = 0;
		if ( iRightBorder < 0 )
			iRightBorder = 0;
		if ( iTopBorder < 0 )
			iTopBorder = 0;
		if ( iBottomBorder < 0 )
			iBottomBorder = 0;
	}
	else
	{
		iLeftBorder = Getx2RCInt(strName.c_str(), "LeftBorder", 0, 0);
		iTopBorder = Getx2RCInt(strName.c_str(), "TopBorder", 0, 0);
		iRightBorder = Getx2RCInt(strName.c_str(), "RightBorder", 0, 0);
		iBottomBorder = Getx2RCInt(strName.c_str(), "BottomBorder", 0, 0);
	}

	iLines_Rows = Getx2RCInt(strName.c_str(), "Lines", 1, 1);
	iMaxLines_Rows = Getx2RCInt(strName.c_str(), "MaxLines", 100, 1);
	if (iLines_Rows > iMaxLines_Rows)
		iLines_Rows = iMaxLines_Rows;

	iLayout = Getx2RCNamedValue(strName.c_str(), "Layout", layoutValues, 0);	
	
	iWrapCount = Getx2RCInt(strName.c_str(), "WrapCount", -1);
	if ( iWrapCount <= 1 )
		iWrapCount = -1;
	if ( iWrapCount == -1 && iLines_Rows > 1 )
		iWrapCount = 5;

	string tmpautoSize = iGetx2RCLine(strName, "AutoSize", "0 0");
	if ( _stricmp(tmpautoSize.c_str(), "0 0") != NULL )
	{
		numTokens = SplitString(tmpautoSize, one, two, three, four, five, six, seven);
		
		bAutoSize = true;

		if ( numTokens >= 2 )
		{
			iAutoSizeSteps = atoi(one.c_str());
			iAutoSizeDelay = atoi(two.c_str());
		}
	}
	else
	{
		bAutoSize = false;
	}

	iDirection = Getx2RCNamedValue(strName.c_str(), "Direction", directionValues, 1);
	iWrapDirection = Getx2RCNamedValue(strName.c_str(), "WrapDirection", wrapdirectionValues, 1);

	if ( iLayout == 0 )
	{
		if ( iDirection == 3 )
			iDirection = 1;
		else if ( iDirection == 4 )
			iDirection = 2;
		if ( iWrapDirection == 3 )
			iWrapDirection = 1;
		else if ( iWrapDirection == 4 )
			iWrapDirection = 2;
	}
	else if ( iLayout == 1 )
	{
		if ( iDirection == 1 )
			iDirection = 3;
		else if ( iDirection == 2 )
			iDirection = 4;
		if ( iWrapDirection == 1 )
			iWrapDirection = 3;
		else if ( iWrapDirection == 2 )
			iWrapDirection = 4;
	}

	iXSpacing = Getx2RCInt(strName.c_str(), "xSpacing", 0);
	iYSpacing = Getx2RCInt(strName.c_str(), "ySpacing", 0);

	bTrueTransparency = Getx2RCBool(strName.c_str(), "TrueTransparency");

	if (refresh)
	{
		for(GhostTrayListIterator gtlit = listGhostTrays.begin(); gtlit != listGhostTrays.end(); gtlit++)
		{
			delete *gtlit;
			*gtlit = NULL;
		}
		listGhostTrays.clear();

		for(GhostButtonListIterator glit = listGhostButtons.begin(); glit != listGhostButtons.end(); glit++)
		{
			delete *glit;
			*glit = NULL;
		}
		listGhostButtons.clear();
	}

	//Create Extra Painting
	StringList extraPainting = GetxRCLineList( "*xTrayOverlayTray" );
	
	for(StringListIterator sit = extraPainting.begin(); sit != extraPainting.end(); sit++)
	{
		numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
		
		if ( numTokens >= 5 )
		{
			GhostTray* newtray = new GhostTray();
		
			newtray->strOrgX = one;
			newtray->strOrgY = two;

			newtray->strOrgWidth = three;
			newtray->strOrgHeight = four;
	
			if ( numTokens >= 6 && IsOS( OS_2KXP ) && (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0) )
			{
				if ( !newtray->pTrayTexture->configure(five.c_str(), "", true) )
				{	
					delete newtray;
					continue;
				}
			}
			else
			{
				if ( !newtray->pTrayTexture->configure(five.c_str(), "", false) )
				{	
					delete newtray;
					continue;
				}
			}
		
			listGhostTrays.insert(listGhostTrays.end(), newtray);
		}
	}
	
	//Create Extra Painting
	extraPainting = GetxRCLineList( "*xTrayOverlayButton" );
	
	for(sit = extraPainting.begin(); sit != extraPainting.end(); sit++)
	{
		//Use Split Helper Strings
		numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
		
		if ( numTokens >= 5 )
		{
			GhostButton* newbutton = new GhostButton();
		
			newbutton->strOrgX = one;
			newbutton->strOrgY = two;

			newbutton->strOrgWidth = three;
			newbutton->strOrgHeight = four;

			PaintTexture *defaultbuttontexture = Create_xTextureClass();

			bool useNormal = false;
			bool useHover = false;
			bool useHidden = false;
			if ( numTokens >= 6 && IsOS( OS_2KXP ) && (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0) )
			{
				defaultbuttontexture->configure(five.c_str(), "Button", TRUE);

				useNormal = newbutton->pNormalButtonTexture->configure(five.c_str(), "NormalButton", TRUE, defaultbuttontexture);
				useHover = newbutton->pHoverButtonTexture->configure(five.c_str(), "HoverButton", TRUE, newbutton->pNormalButtonTexture);
				useHidden = newbutton->pHiddenButtonTexture->configure(five.c_str(), "HiddenButton", TRUE, newbutton->pNormalButtonTexture);
			}
			else
			{
				defaultbuttontexture->configure(five.c_str(), "Button", FALSE);

				useNormal = newbutton->pNormalButtonTexture->configure(five.c_str(), "NormalButton", FALSE, defaultbuttontexture);
				useHover = newbutton->pHoverButtonTexture->configure(five.c_str(), "HoverButton", FALSE, newbutton->pNormalButtonTexture);
				useHidden = newbutton->pHiddenButtonTexture->configure(five.c_str(), "HiddenButton", FALSE, newbutton->pNormalButtonTexture);
			}

			//CleanUp
			Destroy_xTextureClass(defaultbuttontexture);

			if ( !useNormal )
			{
				Destroy_xTextureClass(newbutton->pNormalButtonTexture);
				newbutton->pNormalButtonTexture = NULL;
			}
			if ( !useHover )
			{
				Destroy_xTextureClass(newbutton->pHoverButtonTexture);
				newbutton->pHoverButtonTexture = NULL;
			}
			if ( !useHidden )
			{
				Destroy_xTextureClass(newbutton->pHiddenButtonTexture);
				newbutton->pHiddenButtonTexture = NULL;
			}
		
			if ( useNormal || useHover || useHidden )
				listGhostButtons.insert(listGhostButtons.end(), newbutton);
			else
				delete newbutton;
		}
	}

	int maxUsedIconSize = pNormalIcon->getIconSize();
	if ( pHoverIcon )
		maxUsedIconSize = max(pHoverIcon->getIconSize(), maxUsedIconSize);
	if ( pHiddenIcon )
		maxUsedIconSize = max(pHiddenIcon->getIconSize(), maxUsedIconSize);

	iMaxIconSize = Getx2RCInt(strName.c_str(), "ButtonSize", maxUsedIconSize, 8, 64);

	clrInfoTipColor = Getx2RCColor(strName.c_str(), "ToolTipColor", GetSysColor(COLOR_INFOBK));
	clrInfoTipTextColor = Getx2RCColor(strName.c_str(), "ToolTipTextColor", GetSysColor(COLOR_INFOTEXT));

	bShowToolTip = Getx2RCBool(strName.c_str(), "ShowTooltip", true);

	if (!refresh)
	{
		int maxX = iScreenWidth;
		int maxY = iScreenHeight;
		if ( box )
		{
			RECT rc;
			GetClientRect(box, &rc);
			maxX = rc.right;
			maxY = rc.bottom;
		}

		actualx = Getx2RCCoord(strName.c_str(), "X", 0, maxX, false);
		actualy = Getx2RCCoord(strName.c_str(), "Y", 0, maxY, true);
		
		if ( !bAutoSize )
		{
			actualwidth = Getx2RCDimen(strName.c_str(), "Width", iLeftBorder+iMaxIconSize+iRightBorder, maxX, false);
			actualheight = Getx2RCDimen(strName.c_str(), "Height", iTopBorder+iMaxIconSize+iBottomBorder, maxY, true);
		}
		else
		{
			if ( iAutoSizeSteps != 0 && iAutoSizeDelay != 0 )
			{
				if ( iDirection <= 2 )
				{
					actualwidth = 0;
					actualheight = iTopBorder+iMaxIconSize+iBottomBorder;
				}
				else
				{
					actualwidth = iLeftBorder+iMaxIconSize+iRightBorder;
					actualheight = 0;
				}
			}
			else
			{
				actualwidth = 0;
				actualheight = 0;
			}
		}
		
		Reposition(3, actualx, actualy, actualwidth, actualheight);

		if ( IsOS( OS_2KXP ) )
			bShowInfoTips = Getx2RCBool(strName.c_str(), "ShowInfoTip");
		else
			bShowInfoTips = false;

		if ( bShowToolTip && bShowInfoTips )
		{ 
			// struct specifying control classes to register
			INITCOMMONCONTROLSEX iccex; 

			// INITIALIZE COMMON CONTROLS 
			iccex.dwICC = ICC_WIN95_CLASSES;
			iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);

			InitCommonControlsEx(&iccex);

			if ( !CreateInfoTipWindow() )
				bShowInfoTips = false;
		}
		
		if ( bShowToolTip && !pTrayTooltip )
		{
			pTrayTooltip = Create_xTooltipClass(hWnd, hInstance);
			hWndToolTip = pTrayTooltip->configure(strName.c_str(), "");

			pTrayTooltip->apply("");
		}
	}

	iAlphaTransparency = Getx2RCInt(strName.c_str(), "AlphaTransparency", 255, 0, 255);

	bAlphaMap = Getx2RCBool(strName.c_str(), "AlphaMap");

	string fade = iGetx2RCLine(strName, "CustomAlphaFade", "");
	numTokens = SplitString(fade, one, two, three, four, five, six, seven);

	if (fade.empty())
	{
		bAlphaFade = false;
	}
	else if (numTokens >= 2)
	{
		iAlphaFadeStep = atoi( one.c_str() );
		if (iAlphaFadeStep < 1 || iAlphaFadeStep > 255)
			iAlphaFadeStep = 2;

		iAlphaFadeDelay = atoi( two.c_str() );
		if (iAlphaFadeDelay < 1 || iAlphaFadeDelay > 500)
			iAlphaFadeDelay = 10;

		bAlphaFade = true;
	}

	if (!bAlphaFade)
		bAlphaFade = Getx2RCBool(strName.c_str(), "AlphaFade");

	if ( IsOS(OS_2KXP) && !refresh && (iAlphaTransparency != 255 || bAlphaFade || bAlphaMap) )
	{
		ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

		if (bAlphaFade)
			UpdateWindowTransparency(iUpdateAlpha);
		else
			UpdateWindowTransparency(iAlphaTransparency);
	}
	else if ( !IsOS(OS_2KXP) && !refresh)
	{
		iAlphaTransparency = 255;
		bAlphaFade = false;
		bAlphaMap = false;
	}

	bNormalWindowzOrder = Getx2RCBool(strName.c_str(), "WindowzOrder");

	strBehindWindow = iGetx2RCLine(strName, "BehindWindow", "");

	if (!refresh)
	{
		SetAlwaysOnTop(Getx2RCBool(strName.c_str(), "AlwaysOnTop"), true);
	}
	else
	{
		bBlocked = true;

		if ( bAutoSize )
			AutoSizeFunction();
		
		CreateLayout();

		bBlocked = false;

		if ( !trayLineRemove.empty() ) 
		{
			while ( iLines_RowsAdded > 0 )
			{
				iLines_RowsAdded--;
				PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));

				LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
			}
		}

		if ( !trayRemove.empty() )
		{
			int todo = iVisibleIcons;
			while ( todo > 0 )
			{
				iVisibleIcons--;
				PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons ));
				
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		iVisibleIcons = 0;
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{
			if ( !(*tbit)->bxTrayHideChecked )
			{
				if ( IsOnTrayList((*tbit)->hWndOwner, HideList, false) )
					(*tbit)->bUserHidden = true;

				(*tbit)->bxTrayHideChecked = true;
			}

			if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
				iVisibleIcons++;
		}

		if ( iWrapCount != -1 )
		{
			iLines_RowsAdded = 0;

			int count = iVisibleIcons/iWrapCount;
			int rest = iVisibleIcons%iWrapCount;
			
			int todo = count-iLines_Rows;
		
			if ( count != 0 && rest != 0 )
				todo++;

			while ( todo > 0 && iLines_Rows+iLines_RowsAdded+1 <= iMaxLines_Rows )
			{
				iLines_RowsAdded++;
				PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));

				if ( !trayLineAdd.empty() ) 
					LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

				todo--;
			}
	
			PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));
		}
		
		if ( !trayAdd.empty() )
		{
			int todo = iVisibleIcons;
			while ( todo > 0 )
			{
				PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons-todo+1 ));

				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		CreateLayout();
	}
}
	
void Tray::SetAlwaysOnTop(bool bNewAlwaysOnTop, bool bFirstTime)
{
	if (box)
		return;
	
	bool bRealChange = (bAlwaysOnTop != bNewAlwaysOnTop ? true : false);

	bAlwaysOnTop = bNewAlwaysOnTop;

	if ( (bFirstTime || bRealChange) && !bNormalWindowzOrder )
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, (bAlwaysOnTop || bNormalWindowzOrder || iRealMonitors > 1) ? 0 : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
	}
		
	if ( bAlwaysOnTop )
	{	
		SetWindowPos(hWnd, HWND_TOPMOST,
							-1, -1, -1, -1,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
	else
	{
		SetWindowPos(hWnd, HWND_NOTOPMOST,
							-1, -1, -1, -1,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );	
	}

	if ( !bNormalWindowzOrder )
	{
		if ( !strBehindWindow.empty() && hWndBehindWindow == NULL )
		{
			string one = "";
			string two = "";
			string three = "";
			string four = "";
			string five = "";
			string six = "";
			string seven = "";

			SplitString(strBehindWindow, one, two, three, four, five, six, seven);
		
			if ( _stricmp(one.c_str(), "HWND_TOP") == NULL )
				hWndBehindWindow = HWND_TOP;
			else
				hWndBehindWindow = FindWindowEx2(one.c_str(), two.c_str());

			if ( hWndBehindWindow != NULL )
			{
				SetWindowPos(hWnd, hWndBehindWindow,
									-1, -1, -1, -1,
									SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
			}
		}
		else if ( hWndBehindWindow != NULL )
		{
			SetWindowPos(hWnd, hWndBehindWindow,
								-1, -1, -1, -1,
								SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
		}
	}
}

void Tray::SetAlpha(int newAlphaTransparency)
{
	if (box)
		return;

	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!bLayeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, (bAlwaysOnTop || bNormalWindowzOrder || iRealMonitors > 1) ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			bLayeredFlag = true;

			Repaint();
		}

		iAlphaTransparency = newAlphaTransparency;

		UpdateWindowTransparency(newAlphaTransparency);
	}
}

void Tray::SetAlphaFade(int newAlphaTransparency, int step, int delay)
{
	if (box)
		return;

	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!bLayeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, (bAlwaysOnTop || bNormalWindowzOrder || iRealMonitors > 1) ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			bLayeredFlag = true;

			Repaint();
		}
		
		iUpdateAlpha = iAlphaTransparency;
		
		iAlphaFadeBangEndValue = newAlphaTransparency;

		if ( iUpdateAlpha > iAlphaFadeBangEndValue )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
			SetTimer(hWnd, TIMER_ALPHAFADEDOWNBANG, delay, NULL);
		}
		else if ( iUpdateAlpha < iAlphaFadeBangEndValue )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
			SetTimer(hWnd, TIMER_ALPHAFADEUPBANG, delay, NULL);
		}
	}
}

void Tray::UpdateWindowTransparency(int newAlphaTransparency)
{
	/* First, see if we can get the API call we need. If we've tried
    * once, we don't need to try again. */
	if (!bAlphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		bAlphaInitialized = true;
   
		if (pUpdateLayeredWindow == NULL)
		{
		    ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			iAlphaTransparency = 255;
			bAlphaMap = false;
			bAlphaFade = false;

			bLayeredFlag = false;

			return;
		}
		else
		{
			bLayeredFlag = true;

			SetWindowRgn(hWnd, NULL, true);

			if ( backupBitmap )
			{
				backupBitmap = (HBITMAP) SelectObject(backupDC, oldbackupBMP);
				DeleteObject(backupBitmap);
			}
			if ( backupDC )
				DeleteDC(backupDC);
			if ( oldbackupBMP )
				DeleteObject(oldbackupBMP);

			backupDC = NULL;
			backupBitmap = NULL;
			oldbackupBMP = NULL;
		}
	}

	/* Now, we need to set the 'layered window attributes'. This
	 * is where the alpha values get set. */
	//------------------------------------------------------------------
	RECT rectPosition;
	GetWindowRect(hWnd, &rectPosition);

	int x = rectPosition.left;
	int y = rectPosition.top;
		
	int width = (rectPosition.right-rectPosition.left);
	int height = (rectPosition.bottom-rectPosition.top);

	if ( width <= 0 || height <= 0 )
		return;

	HDC	backgroundDC = CreateCompatibleDC(NULL);
			
	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP	backgroundBitmap = CreateDIBSection( backgroundDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

	// Use Complete Backup for WindowsInternal Repaint (Covering Windows, ...)
	if ( backupBitmap != NULL && backupDC != NULL )
	{
		BitBlt(backgroundDC, 0, 0, width, height, backupDC, 0, 0, SRCCOPY);
	}
	else
	//1----------------------------------------------------------------
	{
	//1----------------------------------------------------------------

		if ( !bAlphaMap && !bTrueTransparency )
		{
			//  =======================================================================
			//  Paint desktop on display DC and then into background buffer
			//  =======================================================================
			PaintDesktopEx(backgroundDC, 0, 0, width, height, x, y, false);
		}
		
		BITMAP bm;
		GetObject(backgroundBitmap, sizeof(BITMAP), &bm);

		BYTE *pPixel = (LPBYTE)bm.bmBits;

		int iSize = width * height;

		for (int j = 0; j < iSize; ++j)
		{
			if ( !bAlphaMap && bTrueTransparency )
			{
				pPixel[0] = 255;
				pPixel[1] = 0;
				pPixel[2] = 255;
				pPixel[3] = 255;
			}
			else if ( !bAlphaMap && !bTrueTransparency )
			{
				pPixel[3] = 255;
			}
			else
			{
				pPixel[0] = 0;
				pPixel[1] = 0;
				pPixel[2] = 0;
				pPixel[3] = 0;
			}

			pPixel+= 4;
		}

		//Normal Background Painting
		pTrayTexture->apply(backgroundDC, 0, 0, width, height);

		if ( !listGhostTrays.empty() )
		{
			for(GhostTrayListIterator glit = listGhostTrays.begin(); glit != listGhostTrays.end(); glit++)
			{
				if ( *glit != NULL )
				{
					int gtx = xParseCoord((*glit)->strOrgX.c_str(), 0, width, false);
					int gty = xParseCoord((*glit)->strOrgY.c_str(), 0, height, true);
					int gtwidth = xParseDimen((*glit)->strOrgWidth.c_str(), width, width, false);
					int gtheight = xParseDimen((*glit)->strOrgHeight.c_str(), height, height, true);

					(*glit)->pTrayTexture->apply(backgroundDC, gtx, gty, gtwidth, gtheight);
				}
				else
				{
					break;
				}
			}
		}

		bool securecapture = false;
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{
			if ( (*tbit) != pDontPaintAddedTrayIcon )
			{
				if ( ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) ) && (*tbit) != pCapture )
					(*tbit)->Paint( backgroundDC, iLeftBorder, iTopBorder, width-iRightBorder, height-iBottomBorder );
				else if ( (*tbit) == pCapture )
					securecapture = true;
			}
		}

		if ( securecapture && ( !pCapture->bUserHidden || (pCapture->bUserHidden && bHiddenVisible) ) )
			pCapture->Paint( backgroundDC, iLeftBorder, iTopBorder, width-iRightBorder, height-iBottomBorder );

		//Prepare Combined Image
		GetObject(backgroundBitmap, sizeof(BITMAP), &bm);

		pPixel = (LPBYTE)bm.bmBits;

		for (j = 0; j < iSize; ++j)
		{
			if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			{
				pPixel[0] = 0;
				pPixel[1] = 0;
				pPixel[2] = 0;
				pPixel[3] = 0;
			}
			else if ( !bAlphaMap )
			{
				pPixel[3] = 255;
			}

			pPixel += 4;
		}
	
		//Create BackupBitmap, if it doesn't exist!
		//----------------------------------------------------------------------
		if ( !bSkipBackup && backupBitmap == NULL && backupDC == NULL )
		{
			backupDC = CreateCompatibleDC(NULL);
			
			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=width;      // Set size you need
			alpha.bmiHeader.biHeight=height;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			backupBitmap = CreateDIBSection( backupDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

			oldbackupBMP = (HBITMAP) SelectObject(backupDC, backupBitmap);

			BitBlt(backupDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);
		}
		//----------------------------------------------------------------------
	//1
	}
	//1}

	POINT ptWindowScreenPosition = {x, y};
	SIZE szWindow = {width, height};

	POINT ptSrc = {0,0};

	BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, newAlphaTransparency, AC_SRC_ALPHA };

	pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, backgroundDC, &ptSrc, 0, &blendPixelFunction, ULW_ALPHA );
	//------------------------------------------------------------------*/

	backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldBMP);
	DeleteDC(backgroundDC);
	DeleteObject(backgroundBitmap);
	DeleteObject(oldBMP);
}

//------------------------------------------------------------
//Show/Hide
//------------------------------------------------------------
void Tray::Show()
{
	if ( bParentDestroyNotificationReceived )
		return;

	if (bAlphaFade && !bVisible)
	{
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		SetTimer(hWnd, TIMER_ALPHAFADEIN, iAlphaFadeDelay, NULL);
	}
	else if ( !bVisible )
	{
		ShowWindow(hWnd, SW_SHOWNOACTIVATE);
	}

	bVisible = true;

	Repaint();

	if ( !trayShow.empty() )
		LSExecute(hWnd, trayShow.c_str(), SW_SHOWNORMAL);
}

void Tray::Hide()
{
	if (bAlphaFade && bVisible)
	{
		iUpdateAlpha = iAlphaTransparency;

		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		SetTimer(hWnd, TIMER_ALPHAFADEOUT, iAlphaFadeDelay, NULL);
	}
	else if ( bVisible )
	{
		ShowWindow(hWnd, SW_HIDE);
	}

	bVisible = false;

	if ( !trayHide.empty() )
		LSExecute(hWnd, trayHide.c_str(), SW_SHOWNORMAL);
}

void Tray::ToggleHiddenIcons()
{
	int changingIcons = 0;
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
	{
		if ( (*tbit)->bUserHidden )
			changingIcons++;
	}

	if ( changingIcons == 0 )
		return;

	if ( bHiddenVisible )
	{
		if ( iWrapCount != -1 )
		{
			int count = changingIcons/iWrapCount;
			int rest = changingIcons%iWrapCount;
			int rest1 = iVisibleIcons%iWrapCount;
	
			int todo = count;

			if ( changingIcons-(count*iWrapCount) >= rest1 )
				todo++;

			while ( todo > 0 && iLines_RowsAdded > 0 )
			{	
				iLines_RowsAdded--;
				PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));
	
				if ( !trayLineRemove.empty() )
					LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
				
				if ( iLines_RowsAdded <= 0 )
				{
					iLines_RowsAdded = 0;
					break;
				}

				todo--;
			}
		}
		
		if ( !trayRemove.empty() )
		{
			while ( changingIcons > 0 )
			{
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);
				changingIcons--;
			}
		}

		bHiddenVisible = false;
	}
	else
	{
		int usechangingIcons = changingIcons;

		if ( iWrapCount != -1 )
		{
			int correct = iVisibleIcons-(iWrapCount*iLines_Rows);
			if ( correct < 0 )
				usechangingIcons = changingIcons+correct;

			if ( usechangingIcons > 0 )
			{
				int count = usechangingIcons/iWrapCount;
				int rest = usechangingIcons%iWrapCount;
				int rest1 = iVisibleIcons%iWrapCount;

				int todo = count;

				if ( usechangingIcons-(count*iWrapCount) >= rest1 )
					todo++;

				while ( todo > 0 && iLines_Rows+iLines_RowsAdded+1 <= iMaxLines_Rows )
				{	
					iLines_RowsAdded++;
					PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows+iLines_RowsAdded ));
		
					if ( !trayLineAdd.empty() )
						LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

					todo--;
				}
			}
		}
		
		if ( !trayAdd.empty() )
		{
			while ( changingIcons > 0 )
			{
				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				changingIcons--;
			}
		}

		bHiddenVisible = true;	
	}
	
	bBlocked = true;

	if ( bAutoSize )
		AutoSizeFunction();
	
	CreateLayout();

	bBlocked = false;
}

//------------------------------------------------------------
//Move/Resize/Reposition
//------------------------------------------------------------
//Type == 1 -> Move
//Type == 2 -> Resize
//Type == 3 -> Reposition

void Tray::Reposition(int type, int newx, int newy, int newwidth, int newheight, int steps, int time, bool repositionBy)
{
	int x = actualx;
	int y = actualy;
	
	int width = actualwidth;
	int height = actualheight;

	if (steps > 0 && time > 0)
	{
		if (time > 50)
			time = 50;

		if (repositionBy)
		{
			if (type != 2)
			{
				newx = x + newx;
				newy = y + newy;
			}
			if (type != 1)
			{
				newwidth = width + newwidth;
				newheight = height + newheight;
			}
		}

		bSkipBackup = true;

		int counter = 0;
		int deltax = newx - x;
		int deltay = newy - y;
		int deltasizex = newwidth - width;
		int deltasizey = newheight - height;

		bool growx;
		bool growy;
		bool growsizex;
		bool growsizey;
		
		if (deltax < 0)
			growx = false;
		else
			growx = true;
		if (deltay < 0)
			growy = false;
		else
			growy = true;
		
		if (deltasizex < 0)
			growsizex = false;
		else
			growsizex = true;
		if (deltasizey < 0)
			growsizey = false;
		else
			growsizey = true;

		int restx = 0;
		int resty = 0;
		int restw = 0;
		int resth = 0;

		div_t div_result;
		div_result = div( deltax, steps );
		deltax = div_result.quot;
		restx = div_result.rem;
		div_result = div( deltay, steps );
		deltay = div_result.quot;
		resty = div_result.rem;

		div_result = div( deltasizex, steps );
		deltasizex = div_result.quot;
		restw = div_result.rem;
		div_result = div( deltasizey, steps );
		deltasizey = div_result.quot;
		resth = div_result.rem;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		if (deltasizex == 0 && growsizex)
			deltasizex = 1;
		else if (deltasizex == 0 && !growsizex)
			deltasizex = -1;
		if (deltasizey == 0 && growsizey)
			deltasizey = 1;
		else if (deltasizey == 0 && !growsizey)
			deltasizey = -1;

		int dynx = x;
		int dyny = y;
		int dynwidth = width;
		int dynheight = height;

		int stopper = 0;
		while (counter <= steps)
		{
			stopper = 0; // Stop if x, y, width and height are finished, stopper = 4
			if ( restx > 0 && deltax > 0 )
			{
				dynx = dynx+deltax+1;
				restx--;
			}
			else if ( restx < 0 && deltax < 0 )
			{
				dynx = dynx+deltax-1;
				restx++;
			}
			else
				dynx = dynx+deltax;

			if (growx && dynx > newx)
			{
				dynx = newx;
				stopper++;
			}
			else if (!growx && dynx < newx)
			{
				dynx = newx;
				stopper++;
			}

			if ( resty > 0 && deltay > 0 )
			{
				dyny = dyny+deltay+1;
				resty--;
			}
			else if ( resty < 0 && deltay < 0 )
			{
				dyny = dyny+deltay-1;
				resty++;
			}
			else
				dyny = dyny+deltay;

			if (growy && dyny > newy)
			{
				dyny = newy;
				stopper++;
			}
			else if (!growy && dyny < newy)
			{
				dyny = newy;
				stopper++;
			}

			if ( restw > 0 && deltasizex > 0 )
			{
				dynwidth = dynwidth+deltasizex+1;
				restw--;
			}
			else if ( restw < 0 && deltasizex < 0 )
			{
				dynwidth = dynwidth+deltasizex-1;
				restw++;
			}
			else
				dynwidth = dynwidth+deltasizex;

			if (growsizex && dynwidth > newwidth)
			{
				dynwidth = newwidth;
				stopper++;
			}
			else if (!growsizex && dynwidth < newwidth)
			{
				dynwidth = newwidth;
				stopper++;
			}

			if ( resth > 0 && deltasizey > 0 )
			{
				dynheight = dynheight+deltasizey+1;
				resth--;
			}
			else if ( resth < 0 && deltasizey < 0 )
			{
				dynheight = dynheight+deltasizey-1;
				resth++;
			}
			else
				dynheight = dynheight+deltasizey;

			if (growsizey && dynheight > newheight)
			{
				dynheight = newheight;
				stopper++;
			}
			else if (!growsizey && dynheight < newheight)
			{
				dynheight = newheight;
				stopper++;
			}

			if ( type == 2 || type == 3 )
				CreateLayout(false, dynwidth, dynheight);
				
			SetWindowPos(hWnd, 0, dynx, dyny, dynwidth, dynheight,
					SWP_NOACTIVATE | (type == 2 ? SWP_NOMOVE : 0) | (type == 1 ? SWP_NOSIZE : 0 ) | SWP_NOZORDER);
				
			if (box != NULL)
				RedrawWindow(box, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
			
			Repaint();

			if (stopper == 4)
				counter = steps;

			counter++;

			Sleep(time);
		}

		//Paint Added TrayIcon with the last Resize Action
		pDontPaintAddedTrayIcon = NULL;

		bSkipBackup = false;

		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			CreateLayout(false, newwidth, newheight);
			
			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);

			Repaint();
		}
		else if ( type == 3 )
		{
			CreateLayout(false, newwidth, newheight);
			
			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);

			Repaint();
		}
	}
	else
	{
		if (repositionBy)
		{
			if (type != 2)
			{
				newx = x + newx;
				newy = y + newy;
			}
			if (type != 1)
			{
				newwidth = width + newwidth;
				newheight = height + newheight;
			}
		}

		//Paint Added TrayIcon with the last Resize Action
		pDontPaintAddedTrayIcon = NULL;

		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			CreateLayout(false, newwidth, newheight);

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);
		}
		else if ( type == 3 )
		{
			CreateLayout(false, newwidth, newheight);

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
		}
	}
}

void Tray::UpdateRegion()
{	
	RECT rectPosition;
	GetClientRect(hWnd, &rectPosition);

	HDC tmpDC = CreateCompatibleDC(NULL);
	HBITMAP tmpBmp = CreateBitmap(rectPosition.right, rectPosition.bottom, 1, bitsperpixel, NULL);
	HBITMAP oldBMP = (HBITMAP) SelectObject(tmpDC, tmpBmp);

	onPaint(tmpDC);
								
	tmpBmp = (HBITMAP) SelectObject(tmpDC, oldBMP);

	HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
	HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(windowRgn, region, NULL, RGN_COPY);

	SetWindowRgn(hWnd, windowRgn, true);

	DeleteObject(windowRgn);
	DeleteObject(region);

	DeleteDC(tmpDC);
	DeleteObject(tmpBmp);
	DeleteObject(oldBMP);
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

TrayButton* Tray::onLButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onLButtonDown(int x, int y, LPARAM lParam)
{  
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button != NULL && !bNoDragnDrop )
	{
		if ( button->iNoDragnDrop == -1 )
		{
			if ( IsOnTrayList(button->hWndOwner, NoDragList, false) )
			{
				pMoveCapture = NULL;
				button->iNoDragnDrop = 0;
			}
			else
			{
				pMoveCapture = button;
				button->iNoDragnDrop = 1;
			}
		}
		else if ( button->iNoDragnDrop == 1 )
		{
			pMoveCapture = button;
		}
	}
	
	if ( (button == NULL || bFireClicksAlways) && !trayLButtonDown.empty() )
	{
		LSExecute(hWnd, trayLButtonDown.c_str(), SW_SHOWNORMAL);
	}
	
	if ( button == NULL && bBorderDrag )
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MOVE | 2, lParam);
	}

	return button;
}

TrayButton* Tray::onLButtonUp(int x, int y)
{	
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button != NULL && !bNoDragnDrop && pMoveCapture != NULL && pMoveCapture != button )
	{	
		if ( button->iNoDragnDrop == -1 )
		{
			if ( IsOnTrayList(button->hWndOwner, NoDragList, false) )
			{
				pMoveCapture = NULL;
				button->iNoDragnDrop = 0;
			}
			else
			{
				button->iNoDragnDrop = 1;
			}
		}
		
		if ( button->iNoDragnDrop == 1 )
		{
			TrayButtonList tmptraybuttons;
			
			for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
			{
				if ( *tbit == button )
				{
					if ( button->iPositionInTray < pMoveCapture->iPositionInTray )
					{
						tmptraybuttons.insert( tmptraybuttons.end(), (pMoveCapture) );
						tmptraybuttons.insert( tmptraybuttons.end(), (button) );
					}
					else if ( button->iPositionInTray > pMoveCapture->iPositionInTray )
					{
						tmptraybuttons.insert( tmptraybuttons.end(), (button) );
						tmptraybuttons.insert( tmptraybuttons.end(), (pMoveCapture) );
					}
				}
				else if ( *tbit != pMoveCapture )
				{
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );
				}
			}
		
			listTrayButtons = tmptraybuttons;
			tmptraybuttons.clear();
			pMoveCapture = NULL;
			CreateLayout();
		}
	}
	
	if ( (button == NULL || bFireClicksAlways) && !trayLButtonUp.empty() )
	{
		LSExecute(hWnd, trayLButtonUp.c_str(), SW_SHOWNORMAL);
	}

	pMoveCapture = NULL;

	return button;
}

TrayButton* Tray::onMButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onMButtonDown(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button && (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		pChangeState = button;
	}
	else if ( button && (0x8000 & GetKeyState(VK_SHIFT)) )
	{
		string info = ("Icon Infos For Correct Hiding:\n\nClassname:\n\"" + GetTaskClass(button->hWndOwner) + "\"\n\nCurrent Windowtext:\n\"" + GetTaskWindowText(button->hWndOwner) + "\"");
		MessageBox(NULL, info.c_str(), "*xTrayHide Info Support!", MB_OK | MB_SETFOREGROUND);
	}
	
	if ( (button == NULL || bFireClicksAlways) && !trayMButtonDown.empty() )
	{
		LSExecute(hWnd, trayMButtonDown.c_str(), SW_SHOWNORMAL);
	}

	return button;
}

TrayButton* Tray::onMButtonUp(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button && button == pChangeState && (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( button->bUserHidden )
		{
			button->bUserHidden = false;
			
			if ( !trayOnIconUnHide.empty() )
			{
				CString exchange = trayOnIconUnHide.c_str();
				exchange.Replace("%[classname]%", GetTaskClass(button->hWndOwner).c_str());
				exchange.Replace("%[windowtext]%", GetTaskWindowText(button->hWndOwner).c_str());
				LSExecute(hWnd, exchange, SW_SHOWNORMAL);
			}
		}
		else
		{
			button->bUserHidden = true;
			if ( !trayRemove.empty() )
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);

			if ( !trayOnIconHide.empty() )
			{
				CString exchange = trayOnIconHide.c_str();
				exchange.Replace("%[classname]%", GetTaskClass(button->hWndOwner).c_str());
				exchange.Replace("%[windowtext]%", GetTaskWindowText(button->hWndOwner).c_str());
				LSExecute(hWnd, exchange, SW_SHOWNORMAL);
			}
		}

		if (pTrayTooltip)
			pTrayTooltip->apply(button->strTooltip.c_str());
	
		if ( bAutoSize )
			AutoSizeFunction();
		
		CreateLayout();
	}
	
	if ( (button == NULL || bFireClicksAlways) && !trayMButtonUp.empty() )
	{
		LSExecute(hWnd, trayMButtonUp.c_str(), SW_SHOWNORMAL);
	}

	pChangeState = NULL;

	return button;
}

TrayButton* Tray::onRButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onRButtonDown(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( (button == NULL || bFireClicksAlways) && !trayRButtonDown.empty() )
	{
		LSExecute(hWnd, trayRButtonDown.c_str(), SW_SHOWNORMAL);
	}

	return button;
}

TrayButton* Tray::onRButtonUp(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( (button == NULL || bFireClicksAlways) && !trayRButtonUp.empty() )
	{
		LSExecute(hWnd, trayRButtonUp.c_str(), SW_SHOWNORMAL);
	}
	
	return button;
}

void Tray::onWheelDown(int x, int y)
{
	if(!trayMWheelDown.empty())
		LSExecute(hWnd, trayMWheelDown.c_str(), SW_SHOWNORMAL);
}

void Tray::onWheelUp(int x, int y)
{
	if(!trayMWheelUp.empty())
		LSExecute(hWnd, trayMWheelUp.c_str(), SW_SHOWNORMAL);
}

void Tray::onMouseEnter()
{
	if(!trayEnter.empty())
		LSExecute(hWnd, trayEnter.c_str(), SW_SHOWNORMAL);

	if ( bShowInfoTips && hWndInfoTip )
		HideInfoTip();

	if (bFocusOnEnter)
	{	
		bSkipButton = true;
		mouse_event( MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MIDDLEDOWN,
						NULL, NULL, NULL, NULL );
		mouse_event( MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MIDDLEUP,
						NULL, NULL, NULL, NULL );
	}
}

void Tray::onMouseLeave()
{
	if (!trayLeave.empty())
		LSExecute(hWnd, trayLeave.c_str(), SW_SHOWNORMAL);
}

TrayButton* Tray::onMouseMove(int x, int y)
{
	if (!bMouseInside)
	{
		bMouseInside = true;
		onMouseEnter();
		SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
	}

	TrayButton *button = ButtonAtPoint(x, y);

	return button;
}

void Tray::Repaint()
{	
	if ( backupBitmap )
	{
		backupBitmap = (HBITMAP) SelectObject(backupDC, oldbackupBMP);
		DeleteObject(backupBitmap);
	}
	if ( backupDC )
		DeleteDC(backupDC);
	if ( oldbackupBMP )
		DeleteObject(oldbackupBMP);

	backupDC = NULL;
	backupBitmap = NULL;
	oldbackupBMP = NULL;

	//Attention!! Background MUST exist
	if ( bLayeredFlag && IsOS(OS_2KXP) )
	{
		UpdateWindowTransparency(iAlphaTransparency);
	}
	else
	{
		RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);

		if ( bTrueTransparency )
			UpdateRegion();
	}
}

void Tray::onPaint(HDC hDC)
{	
	RECT rectPosition;
	GetWindowRect(hWnd, &rectPosition);
		
	int x = rectPosition.left;
	int y = rectPosition.top;

	int width = (rectPosition.right-rectPosition.left);
	int height = (rectPosition.bottom-rectPosition.top);

	if (width <= 0 || height <= 0)
		return;

	HDC backgroundDC = NULL;
	HBITMAP backgroundBitmap = NULL;
	HBITMAP oldBMP = NULL;

	// Use Complete Backup for WindowsInternal Repaint (Covering Windows, ...)
	if ( backupBitmap != NULL && backupDC != NULL )
	{
		BitBlt(hDC, 0, 0, width, height, backupDC, 0, 0, SRCCOPY);
	}
	else
	//1----------------------------------------------------------------
	{
	//1----------------------------------------------------------------
	
		backgroundDC = CreateCompatibleDC(hDC);
		backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
		oldBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

		//True Transparency
		if (bTrueTransparency)
		{
			::SetBkColor(backgroundDC, RGB(255, 0, 255));
			::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &rectPosition, NULL, 0, NULL);
		}
		//PrePaint Background (Fake Transparency)
		else
		{
			//  =======================================================================
			//  Paint desktop on display DC and then into background buffer
			//  =======================================================================
			PaintDesktopEx(backgroundDC, 0, 0, width, height, x, y, false);
		}
				
		//Normal Background Painting
		pTrayTexture->apply(backgroundDC, 0, 0, width, height);

		if ( !listGhostTrays.empty() )
		{
			for(GhostTrayListIterator glit = listGhostTrays.begin(); glit != listGhostTrays.end(); glit++)
			{
				if ( *glit != NULL )
				{
					int gtx = xParseCoord((*glit)->strOrgX.c_str(), 0, width, false);
					int gty = xParseCoord((*glit)->strOrgY.c_str(), 0, height, true);
					int gtwidth = xParseDimen((*glit)->strOrgWidth.c_str(), width, width, false);
					int gtheight = xParseDimen((*glit)->strOrgHeight.c_str(), height, height, true);

					(*glit)->pTrayTexture->apply(backgroundDC, gtx, gty, gtwidth, gtheight);
				}
				else
				{
					break;
				}
			}
		}

		bool securecapture = false;
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{
			if ( (*tbit) != pDontPaintAddedTrayIcon )
			{
				if ( ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) ) && (*tbit) != pCapture )
					(*tbit)->Paint( backgroundDC, iLeftBorder, iTopBorder, width-iRightBorder, height-iBottomBorder );
				else if ( (*tbit) == pCapture )
					securecapture = true;
			}
		}

		if ( securecapture && ( !pCapture->bUserHidden || (pCapture->bUserHidden && bHiddenVisible) ) )
			pCapture->Paint( backgroundDC, iLeftBorder, iTopBorder, width-iRightBorder, height-iBottomBorder );
		
		// blt the double buffer to the display
		BitBlt(hDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

		//Create BackupBitmap, if it doesn't exist!
		//----------------------------------------------------------------------
		if ( !bSkipBackup && backupBitmap == NULL && backupDC == NULL )
		{
			backupDC = CreateCompatibleDC(hDC);
			backupBitmap = CreateCompatibleBitmap(hDC, width, height);
			oldbackupBMP = (HBITMAP) SelectObject(backupDC, backupBitmap);

			BitBlt(backupDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);
		}
		//----------------------------------------------------------------------

	//1----------------------------------------------------------------
	}
	//1----------------------------------------------------------------

	if ( backgroundBitmap != NULL && backgroundDC != NULL )
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldBMP);
		DeleteDC(backgroundDC);
		DeleteObject(backgroundBitmap);
		DeleteObject(oldBMP);
	}
}

void Tray::onTimer(int timerID)
{
	switch(timerID)
	{	
		case TIMER_MOUSETRACK:
		{
			POINT pt;
			GetCursorPos(&pt);
			
			RECT rc;
			GetWindowRect(hWnd, &rc);

			if ( bTrueTransparency )
			{
				HRGN hrgn = CreateRectRgn(0,0,0,0);
				int regionType = GetWindowRgn(hWnd, hrgn);
				if (regionType != ERROR) 
				{ 
					/* hrgn contains window region */ 
					ScreenToClient(hWnd, &pt);
					if(!PtInRegion(hrgn, pt.x, pt.y))
					{
						KillTimer(hWnd, TIMER_MOUSETRACK);

						bMouseInside = false;
						onMouseLeave();	
						if ( bShowToolTip && pTrayTooltip )
							pTrayTooltip->apply("");
					
						if (pCapture)
							pCapture->bUseHover = false;

						Repaint();	

						pCapture = NULL;
					
						DeleteObject(hrgn); /* finished with region */

						break;
					}
				}

				DeleteObject(hrgn); /* finished with region */
			}

			GetCursorPos(&pt);

			if(!PtInRect(&rc, pt))
			{
				KillTimer(hWnd, TIMER_MOUSETRACK);

				bMouseInside = false;
				onMouseLeave();	
				if ( bShowToolTip && pTrayTooltip )
					pTrayTooltip->apply("");
			
				if (pCapture)
					pCapture->bUseHover = false;

				Repaint();	

				pCapture = NULL;
				break;
			}

			ScreenToClient(hWnd, &pt);
			TrayButton *button = ButtonAtPoint( pt.x, pt.y );

			if ( button && button != pCapture )
			{
				if ( bShowToolTip && pTrayTooltip )
				{
					string strTooltip = button->strTooltip;
					if ( button->bUserHidden )
						strTooltip.append("\nCurrent State In xTray: Hidden");
					pTrayTooltip->apply(strTooltip.c_str());
				}
							
				if (pCapture)
					pCapture->bUseHover = false;

				button->bUseHover = true;
				pCapture = button;
				Repaint();
			}
			else if ( button == NULL && pCapture != NULL )
			{
				if ( bShowToolTip && pTrayTooltip )
					pTrayTooltip->apply("");
			
				pCapture->bUseHover = false;
				pCapture = NULL;
				Repaint();
			}
		}
		break;

		case TIMER_ALPHAFADEIN:
		{
			if (iUpdateAlpha <= iAlphaTransparency-iAlphaFadeStep)
			{
				iUpdateAlpha += iAlphaFadeStep;
				UpdateWindowTransparency(iUpdateAlpha);
				ShowWindow(hWnd, SW_SHOWNOACTIVATE);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (iUpdateAlpha != iAlphaTransparency)
					UpdateWindowTransparency(iAlphaTransparency);
			}
		}
		break;

		case TIMER_ALPHAFADEOUT:
		{
			if (iUpdateAlpha >= iAlphaFadeStep)
			{
				iUpdateAlpha -= iAlphaFadeStep;
				UpdateWindowTransparency(iUpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				iUpdateAlpha = 0;
				ShowWindow(hWnd, SW_HIDE);
			}
		}
		break;

		case TIMER_CLEAN:
		{
			bool clean = true;
			bool wascleaned = false;
			while ( clean )
			{
				clean = IconCleanUp();

				if ( clean )
				{
					wascleaned = true;

					if ( !trayRemove.empty() )
					{
						iVisibleIcons--;
						PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons ));

						LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);	
					}
				}
			}

			if ( wascleaned )
			{
				if ( bAutoSize )
					AutoSizeFunction();
			
				CreateLayout();
			}
		}
		break;

		case TIMER_INFOTIPTIMEOUT:
		{
			KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
			bool secure = false;
			for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
			{
				if ( (*tbit) == UpdateConfig.button )
					secure = true;
			}

			if ( secure )
			{
				if ( IsWindow(UpdateConfig.button->hWndOwner) )
					ShowInfoTip(UpdateConfig.button, UpdateConfig.x, UpdateConfig.y);
			}
		}
		break;

		case INFOTIMER_AUTOHIDE:
		{
			HideInfoTip();	
		}	
		break;

		case TIMER_SECURESTART:
		{
			KillTimer(hWnd, TIMER_SECURESTART);
			bInfoTipsEnabled = true;
			SetTimer(hWnd, TIMER_CLEAN, iCleanUpInterval, 0);
		}
		break;

		case TIMER_TRAYADD:
		{
			KillTimer(hWnd, TIMER_TRAYADD);
			if ( !trayAdd.empty() )
			{
				while ( iInternalAddCount > 0 )
				{
					if ( pAddedTrayIcon != NULL )
					{
						if ( IsOnTrayList(pAddedTrayIcon->hWndOwner, HideList, false) )
						{
							pAddedTrayIcon->bUserHidden = true;
						}
						else
						{
							iVisibleIcons++;
							PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons ));
							LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
						}
					}
					
					iInternalAddCount--;
				}
			}

			iInternalAddCount = 0;
			pAddedTrayIcon = NULL;

			if ( bAutoSize )
				AutoSizeFunction();
				
			CreateLayout();
		}
		break;


		case TIMER_TRAYREMOVE:
		{
			KillTimer(hWnd, TIMER_TRAYREMOVE);
			if ( !trayRemove.empty() )
			{
				if ( bRemovedTrayIcon )
				{
					iVisibleIcons--;
					PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( iVisibleIcons ));
				}
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);	
			}

			bRemovedTrayIcon = false;
						
			if ( bAutoSize )
				AutoSizeFunction();
			
			CreateLayout();
		}
		break;

		case TIMER_ALPHAFADEUPBANG:
		{
			if (iUpdateAlpha < iAlphaFadeBangEndValue-iAlphaFadeBangStep)
			{
				iUpdateAlpha += iAlphaFadeStep;
				UpdateWindowTransparency(iUpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
				if (iUpdateAlpha != iAlphaFadeBangEndValue)
					UpdateWindowTransparency(iAlphaFadeBangEndValue);

				iAlphaTransparency = iAlphaFadeBangEndValue;
			}
		}
		
		break;

		case TIMER_ALPHAFADEDOWNBANG:
		{
			if (iUpdateAlpha > iAlphaFadeBangEndValue)
			{
				iUpdateAlpha -= iAlphaFadeStep;
				UpdateWindowTransparency(iUpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
				if (iUpdateAlpha != iAlphaFadeBangEndValue)
					UpdateWindowTransparency(iAlphaFadeBangEndValue);

				iAlphaTransparency = iAlphaFadeBangEndValue;
			}
		}

		break;
	}
}

void Tray::onWindowPosChanging(WINDOWPOS *windowPos)
{
	if ( !bNormalWindowzOrder && !(windowPos->x == -1 && windowPos->y == -1 && windowPos->cx == -1 && windowPos->cy == -1) && (windowPos->flags & SWP_NOZORDER) == 0 )
	{
		if ( !bAlwaysOnTop ) 
		{
			windowPos->flags = windowPos->flags | SWP_NOZORDER;
		}
		else if ( !strBehindWindow.empty() && hWndBehindWindow == NULL )
		{
			string one = "";
			string two = "";
			string three = "";
			string four = "";
			string five = "";
			string six = "";
			string seven = "";

			SplitString(strBehindWindow, one, two, three, four, five, six, seven);
		
			if ( _stricmp(one.c_str(), "HWND_TOP") == NULL )
				hWndBehindWindow = HWND_TOP;
			else
				hWndBehindWindow = FindWindowEx2(one.c_str(), two.c_str());

			if ( hWndBehindWindow != NULL )
			{
				windowPos->hwndInsertAfter = hWndBehindWindow;
			}
		}
		else if ( hWndBehindWindow != NULL )
		{
			windowPos->hwndInsertAfter = hWndBehindWindow;
		}
	}
}

void Tray::onMove(int x, int y)
{
	PrefixedLSSetVariable(strName, "CurrentX", IntToString( x ));
	PrefixedLSSetVariable(strName, "CurrentY", IntToString( y ));
	
	actualx = x;
	actualy = y;

	if ( !bTrueTransparency )
		Repaint();

	if( !trayMove.empty() )
		LSExecute(hWnd, trayMove.c_str(), SW_SHOWNORMAL);
}

void Tray::onSize(int width, int height)
{
	PrefixedLSSetVariable(strName, "CurrentWidth", IntToString( width ));
	PrefixedLSSetVariable(strName, "CurrentHeight", IntToString( height ));

	actualwidth = width;
	actualheight = height;

	Repaint();

	//Set new Rect for ToolTip Tracker
	if ( bShowToolTip && pTrayTooltip )
		pTrayTooltip->apply("", true);
			
	if( !trayResize.empty() )
		LSExecute(hWnd, trayResize.c_str(), SW_SHOWNORMAL);
}

BOOL Tray::onWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	switch ( nMessage )
	{
		case LM_SYSTRAY:
		{
			switch ( wParam )
			{
				case NIM_ADD:
				{
					if ( IconAdd( (NOTIFYICONDATA*)lParam ) )
					{		
						if ( !bBlocked )
						{
							iInternalAddCount++;
							SetTimer(hWnd, TIMER_TRAYADD, 1, 0);
						}
						else
						{
							pDontPaintAddedTrayIcon = NULL;
							pAddedTrayIcon = NULL;
						}
						lResult = TRUE;
					}

					return TRUE;
				}

				case NIM_MODIFY:
				{
					if ( IconModify( (NOTIFYICONDATA*)lParam ) )
					{
						Repaint();
						lResult = TRUE;
					}
					
					return TRUE;
				}

				case NIM_DELETE:
				{
					int result = IconDelete( (NOTIFYICONDATA*)lParam );

					if ( result == 1 )
					{
						SetTimer(hWnd, TIMER_TRAYREMOVE, 1, 0);
						lResult = TRUE;
					}
					else if ( result == -1 )
					{
						lResult = TRUE;
					}

					return TRUE;
				}

				case NIM_SETFOCUS:
				{
					lResult = TRUE;
					return TRUE;
				}

				case NIM_SETVERSION:
				{
					lResult = FALSE;
					return TRUE;
				}
			
				default:
				{
					lResult = FALSE;
					return TRUE;
				}
			}
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_DESTROY:
		{
			if(box)
			{
				if (pTrayTooltip)
				{
					Destroy_xTooltipClass(pTrayTooltip);
					pTrayTooltip = 0;
				}

				if (hWnd)
				{
					KillTimer(hWnd, TIMER_MOUSETRACK);
					KillTimer(hWnd, TIMER_ALPHAFADEIN);
					KillTimer(hWnd, TIMER_ALPHAFADEOUT);
					KillTimer(hWnd, TIMER_CLEAN);
					KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
					KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
					KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
				}

				box = NULL;
				Hide();
				
				bParentDestroyNotificationReceived = true;
			}
			
			return false;
		}

		case WM_LBUTTONDBLCLK:
		{
			TrayButton *button = onLButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			TrayButton *button = onLButtonDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam), lParam );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_LBUTTONUP:
		{
			TrayButton *button = onLButtonUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONDBLCLK:
		{
			TrayButton *button = onMButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			if ( bSkipButton )
				return true;

			TrayButton *button = onMButtonDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONDBLCLK:
		{
			TrayButton *button = onRButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONUP:
		{
			if ( bSkipButton )
			{
				bSkipButton = false;
				return true;
			}

			TrayButton *button = onMButtonUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			TrayButton *button = onRButtonDown( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONUP:
		{
			TrayButton *button = onRButtonUp( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MOUSEWHEEL:
		{
			if ( (short)(HIWORD(wParam)) < 0 )
			{
				onWheelDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
				return true;
			}
			else
			{
				onWheelUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
				return true;
			}
		}

		case WM_MOUSEMOVE:
		{
			if ( hWndToolTip )
			{
				MSG hintMessage = {hWnd, nMessage, wParam, lParam};
				SendMessage( hWndToolTip, (UINT) TTM_RELAYEVENT, 0, (LPARAM) (LPMSG) &hintMessage ); 
			}

			TrayButton *button = onMouseMove( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->ButtonClick(nMessage, wParam, lParam);

			return true;
		}

		case WM_PAINT:
		{
			if (bVisible)
			{
				PAINTSTRUCT ps;

				HDC hDC;

				hDC = BeginPaint( hWnd, &ps );

				onPaint( hDC );
			
				EndPaint( hWnd, &ps );
			}

			return true;
		}

		case WM_WINDOWPOSCHANGING:
		{
			onWindowPosChanging( (WINDOWPOS *) lParam );
			return false;
		}

		case WM_MOVE:
		{
			onMove( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			return true;
		}

		case WM_SIZE:
		{
			onSize( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			return true;
		}

		case WM_TIMER:
		{
			onTimer( (int) wParam );
			return true;
		}

		case WM_ENDSESSION:
		case WM_QUERYENDSESSION:
		{
			lResult = SendMessage(GetLitestepWnd(), nMessage, wParam, lParam);
			return true;
		}
	}

	return false;
}

TrayButton *Tray::ButtonAtPoint( int x, int y )
{
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
	{
		if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
		{
			if ( (*tbit)->HasPoint(x, y) )
				return (*tbit);
		}
	}

	return NULL;
}

LRESULT WINAPI Tray::WindowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Tray *tray = (Tray *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		tray = (Tray *) pcs->lpCreateParams;

		tray->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) tray);
	}

	if (tray)
	{
		LRESULT lResult;

		if (tray->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

void Tray::CreateLayout(bool clearBackground, int width, int height)
{
	if ( bInCriticalProcess )
		return;

	bInCriticalProcess = true;

	if ( width == -1 || height == -1 )
	{
		width = actualwidth;
		height = actualheight;
	}

	iVisibleIcons = 0;
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
	{
		if ( !(*tbit)->bxTrayHideChecked )
		{
			if ( IsOnTrayList((*tbit)->hWndOwner, HideList, false) )
				(*tbit)->bUserHidden = true;

			(*tbit)->bxTrayHideChecked = true;
		}

		(*tbit)->iFixedPositionInTray = PosOnTrayList((*tbit)->hWndOwner, SortList, 100);

		if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
			iVisibleIcons++;
	}

	SortTrayIcons(listTrayButtons);

	if ( iStartWithPosition > iVisibleIcons )
		iStartWithPosition = max(0, iVisibleIcons-1);

	int buttonCount = iVisibleIcons;

	PrefixedLSSetVariable(strName, "CurrentIconCount", IntToString( buttonCount ));
	
	if (buttonCount <= 0)
	{
		PrefixedLSSetVariable(strName, "CurrentLines", IntToString( iLines_Rows ));

		bInCriticalProcess = false;
	
		if ( clearBackground )
			Repaint();

		iBackupExportedButtonCount = buttonCount;

		if ( bHideIfEmpty && !bAutoHidden && bVisible)
		{
			bAutoHidden = true;
			Hide();
		}

		return;
	}
	else if ( bHideIfEmpty && bAutoHidden && !bVisible )
	{
		bAutoHidden = false;
		Show();
	}

	if (iLayout == 0)
	{
		//Start Horizontal Version
		//-----------------------------------------------------------------------
		int fireLineCount = 0;

		if ( iWrapCount != -1 && !bBlocked )
		{
			if ( buttonCount/iWrapCount >= iLines_Rows+iLines_RowsAdded && buttonCount%iWrapCount == 1 && iBackupExportedButtonCount < buttonCount )
			{
				iLines_RowsAdded++;
				if ( iLines_Rows+iLines_RowsAdded <= iMaxLines_Rows )
				{	
					if ( !trayLineAdd.empty() )
						fireLineCount = 1;
				}
				else if ( iLines_Rows+iLines_RowsAdded > iMaxLines_Rows)
				{
					iLines_RowsAdded--;
					iWrapCount++;
				}
			}
			else if ( iLines_RowsAdded >= 1 && buttonCount/iWrapCount >= iLines_Rows && buttonCount%iWrapCount == 0 && iBackupExportedButtonCount > buttonCount)
			{
				iLines_RowsAdded--;
				if ( !trayLineRemove.empty() )
					fireLineCount = -1;
			}
		}

		int realLines = min(iLines_Rows+iLines_RowsAdded, iMaxLines_Rows);

		PrefixedLSSetVariable(strName, "CurrentLines", IntToString( realLines ));

		if ( fireLineCount == -1 )
		{
			LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);

			bInCriticalProcess = false;

			iBackupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}
		else if ( fireLineCount == 1 )
		{
			LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

			bInCriticalProcess = false;

			iBackupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}

		int x = 0;
		int y = 0;

		int buttoncounter = 1;
		int hcounter = 1;
		int vcounter = 1;

		int scrollcounter = 0;

		TrayButtonList tmptraybuttons;
		tmptraybuttons.clear();

		// set placement for each button
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{
			if ( iStartWithPosition > scrollcounter )
			{
				scrollcounter++;

				if ( !bLoopingTrayIcons )
					(*tbit)->SetButtonLayout( -1, -1, -1, -1 );
				else if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );

				continue;
			}
			else
			{
				if ( (*tbit)->bUserHidden && !bHiddenVisible )
					continue;
				scrollcounter++;
			}

			if (iDirection == 1) //Right
				x = iLeftBorder + (hcounter-1) * (iMaxIconSize + iXSpacing);
			else if (iDirection == 2) // Left
				x = (width - iRightBorder) - hcounter * iMaxIconSize - (hcounter-1) * iXSpacing;
			
			if ( iWrapDirection == 1) //Bottom
				y = iTopBorder + (vcounter-1) * (iMaxIconSize + iYSpacing);
			else if ( iWrapDirection == 2) //Top
				y = (height - iBottomBorder) - vcounter * iMaxIconSize - (vcounter-1) * iYSpacing;

			(*tbit)->SetButtonLayout( x, y, iMaxIconSize, iMaxIconSize );

			(*tbit)->iPositionInTray = buttoncounter;

			if ( iWrapCount != -1 && buttoncounter%iWrapCount == 0 )
			{
				hcounter = 0;
				vcounter++;
			}
			hcounter++;
			buttoncounter++;
		}

		if ( bLoopingTrayIcons && !tmptraybuttons.empty() )
		{
			for (TrayButtonListIterator tbit = tmptraybuttons.begin(); tbit != tmptraybuttons.end(); ++tbit)
			{
				if (iDirection == 1) //Right
					x = iLeftBorder + (hcounter-1) * (iMaxIconSize + iXSpacing);
				else if (iDirection == 2) // Left
					x = (width - iRightBorder) - hcounter * iMaxIconSize - (hcounter-1) * iXSpacing;
				
				if ( iWrapDirection == 1) //Bottom
					y = iTopBorder + (vcounter-1) * (iMaxIconSize + iYSpacing);
				else if ( iWrapDirection == 2) //Top
					y = (height - iBottomBorder) - vcounter * iMaxIconSize - (vcounter-1) * iYSpacing;

				(*tbit)->SetButtonLayout( x, y, iMaxIconSize, iMaxIconSize );

				(*tbit)->iPositionInTray = buttoncounter;

				if ( iWrapCount != -1 && buttoncounter%iWrapCount == 0 )
				{
					hcounter = 0;
					vcounter++;
				}
				hcounter++;
				buttoncounter++;
			}
		}
		//-----------------------------------------------------------------------
		//End Horizontal Version
	}
	else if (iLayout == 1)
	{
		//Start Vertical Version
		//-----------------------------------------------------------------------
		int fireLineCount = 0;

		if ( iWrapCount != -1 && !bBlocked )
		{
			if ( buttonCount/iWrapCount >= iLines_Rows+iLines_RowsAdded && buttonCount%iWrapCount == 1 && iBackupExportedButtonCount < buttonCount )
			{
				iLines_RowsAdded++;
				if ( iLines_Rows+iLines_RowsAdded <= iMaxLines_Rows )
				{	
					if ( !trayLineAdd.empty() )
						fireLineCount = 1;
				}
				else if ( iLines_Rows+iLines_RowsAdded > iMaxLines_Rows)
				{
					iLines_RowsAdded--;
					iWrapCount++;
				}
			}
			else if ( iLines_RowsAdded >= 1 && buttonCount/iWrapCount >= iLines_Rows && buttonCount%iWrapCount == 0 && iBackupExportedButtonCount > buttonCount)
			{
				iLines_RowsAdded--;
				if ( !trayLineRemove.empty() )
					fireLineCount = -1;
			}
		}

		int realLines = min(iLines_Rows+iLines_RowsAdded, iMaxLines_Rows);

		PrefixedLSSetVariable(strName, "CurrentLines", IntToString( realLines ));

		if ( fireLineCount == -1 )
		{
			LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
			
			bInCriticalProcess = false;

			iBackupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}
		else if ( fireLineCount == 1 )
		{
			LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);
			
			bInCriticalProcess = false;

			iBackupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}

		int x = 0;
		int y = 0;

		int buttoncounter = 1;
		int hcounter = 1;
		int vcounter = 1;

		int scrollcounter = 0;

		TrayButtonList tmptraybuttons;
		tmptraybuttons.clear();

		// set placement for each button
		for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
		{	
			if ( iStartWithPosition > scrollcounter )
			{
				scrollcounter++;

				if ( !bLoopingTrayIcons )
					(*tbit)->SetButtonLayout( -1, -1, -1, -1 );
				else if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );

				continue;
			}
			else
			{
				if ( (*tbit)->bUserHidden && !bHiddenVisible )
					continue;
				scrollcounter++;
			}

			if ( iDirection == 3 ) //bottom
				y = iTopBorder + (vcounter-1) * (iMaxIconSize + iYSpacing);
			else if ( iDirection == 4)  //top
				y = (height - iBottomBorder) - vcounter * iMaxIconSize - (vcounter-1) * iYSpacing;
	
			if ( iWrapDirection == 3 ) //right
				x = iLeftBorder + (hcounter-1) * (iMaxIconSize + iXSpacing);
			else if ( iWrapDirection == 4 ) //left
				x = (width - iRightBorder) - hcounter * iMaxIconSize - (hcounter-1) * iXSpacing;

			(*tbit)->SetButtonLayout( x, y, iMaxIconSize, iMaxIconSize );

			(*tbit)->iPositionInTray = buttoncounter;

			if ( iWrapCount != -1 && buttoncounter%iWrapCount == 0 )
			{
				hcounter++;
				vcounter = 0;
			}
			vcounter++;
			buttoncounter++;
		}

		if ( bLoopingTrayIcons && !tmptraybuttons.empty() )
		{
			for (TrayButtonListIterator tbit = tmptraybuttons.begin(); tbit != tmptraybuttons.end(); ++tbit)
			{	
				if ( iDirection == 3 ) //bottom
					y = iTopBorder + (vcounter-1) * (iMaxIconSize + iYSpacing);
				else if ( iDirection == 4)  //top
					y = (height - iBottomBorder) - vcounter * iMaxIconSize - (vcounter-1) * iYSpacing;
		
				if ( iWrapDirection == 3 ) //right
					x = iLeftBorder + (hcounter-1) * (iMaxIconSize + iXSpacing);
				else if ( iWrapDirection == 4 ) //left
					x = (width - iRightBorder) - hcounter * iMaxIconSize - (hcounter-1) * iXSpacing;

				(*tbit)->SetButtonLayout( x, y, iMaxIconSize, iMaxIconSize );

				(*tbit)->iPositionInTray = buttoncounter;

				if ( iWrapCount != -1 && buttoncounter%iWrapCount == 0 )
				{
					hcounter++;
					vcounter = 0;
				}
				vcounter++;
				buttoncounter++;
			}
		}
		//-----------------------------------------------------------------------
		//End Vertical Version
	}

	bInCriticalProcess = false;

	if ( clearBackground )
		Repaint();

	iBackupExportedButtonCount = buttonCount;
}

void Tray::ScrollIcons(bool next, int amount)
{
	if ( !bAutoSize )
	{
		if ( !bLoopingTrayIcons )
		{
			if (next)
			{
				if ( iStartWithPosition+amount < iVisibleIcons )
					iStartWithPosition = iStartWithPosition+amount;
			}
			else
			{
				if ( iStartWithPosition-amount < 0 )
					iStartWithPosition = 0;
				else
					iStartWithPosition = iStartWithPosition-amount;
			}	
		}
		else
		{
			if (next)
			{
				iStartWithPosition = iStartWithPosition+amount;
				if ( iStartWithPosition >= iVisibleIcons )
					iStartWithPosition = 0;
			}
			else
			{
				iStartWithPosition = iStartWithPosition-amount;
				if ( iStartWithPosition < 0 )
					iStartWithPosition = iVisibleIcons-1;
			}
		}

		CreateLayout();
	}
}

void Tray::AutoSizeFunction()
{	
	int width = actualwidth;
	int height = actualheight;

	iVisibleIcons = 0;
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit)
	{
		if ( !(*tbit)->bxTrayHideChecked )
		{
			if ( IsOnTrayList((*tbit)->hWndOwner, HideList, false) )
				(*tbit)->bUserHidden = true;

			(*tbit)->bxTrayHideChecked = true;
		}
	
		if ( !(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible) )
			iVisibleIcons++;
	}

	int hcounter = 0;
	int vcounter = 0;

	int autoX = actualx;
	int autoY = actualy;
	int autoWidth = actualwidth;
	int autoHeight = actualheight;

	if (iLayout == 0)
	{
		if ( iWrapCount != -1 )
		{
			hcounter = iVisibleIcons%iWrapCount;	
			vcounter = min(iVisibleIcons/iWrapCount, iMaxLines_Rows);
			if ( vcounter > 1 && hcounter == 1 )
				CreateLayout(false);
		}
		else
		{
			hcounter = iVisibleIcons;
		}
		
		autoX = actualx;
		autoY = actualy;
	
		autoWidth = width;
		autoHeight = height;

		if (iWrapCount == -1 || ( iWrapCount != -1 && iVisibleIcons <= iWrapCount ) )
		{
			autoWidth = iLeftBorder+(iVisibleIcons-1)*iXSpacing+iVisibleIcons*iMaxIconSize+iRightBorder;
			autoHeight = iTopBorder+iMaxIconSize+iBottomBorder;
		}
		else if ( vcounter >= 1 )
		{
			autoWidth = iLeftBorder+(iWrapCount-1)*iXSpacing+iWrapCount*iMaxIconSize+iRightBorder;
			
			if ( iVisibleIcons%iWrapCount == 0 || vcounter >= iMaxLines_Rows )
			{
				autoHeight = iTopBorder+(vcounter-1)*iYSpacing+vcounter*iMaxIconSize+iBottomBorder;
			}
			else
			{
				autoHeight = iTopBorder+vcounter*iYSpacing+(vcounter+1)*iMaxIconSize+iBottomBorder;
			}
		}

		if ( iDirection == 2 )
		{
			autoX = actualx-(autoWidth-width);
		}
		if ( iWrapDirection == 2 )
		{
			autoY = actualy-(autoHeight-height);
		}
			
		if ( actualx != autoX || actualy != autoY || actualwidth != autoWidth || actualheight != autoHeight )
		{
			Reposition(3, autoX, autoY, autoWidth, autoHeight, iAutoSizeSteps, iAutoSizeDelay);
		}
		else
		{
			pDontPaintAddedTrayIcon = NULL;
			pAddedTrayIcon = NULL;
		}
	}
	else if (iLayout == 1)
	{
		if ( iWrapCount != -1 )
		{
			vcounter = iVisibleIcons%iWrapCount;
			hcounter = min(iVisibleIcons/iWrapCount, iMaxLines_Rows);
			if ( hcounter > 1 && vcounter == 1 )
				CreateLayout(false);
		}
		else
		{
			vcounter = iVisibleIcons;
		}

		autoX = actualx;
		autoY = actualy;
	
		autoWidth = width;
		autoHeight = height;

		if (iWrapCount == -1 || ( iWrapCount != -1 && iVisibleIcons <= iWrapCount ) )
		{
			autoWidth = iLeftBorder+iMaxIconSize+iRightBorder;
			autoHeight = iTopBorder+(iVisibleIcons-1)*iYSpacing+iVisibleIcons*iMaxIconSize+iBottomBorder;
		}
		else if ( hcounter >= 1 )
		{
			if ( iVisibleIcons%iWrapCount == 0 || hcounter >= iMaxLines_Rows)
			{
				autoWidth = iLeftBorder+(hcounter-1)*iXSpacing+hcounter*iMaxIconSize+iRightBorder;
			}
			else
			{
				autoWidth = iLeftBorder+hcounter*iXSpacing+(hcounter+1)*iMaxIconSize+iRightBorder;
			}
			autoHeight = iTopBorder+(iWrapCount-1)*iYSpacing+iWrapCount*iMaxIconSize+iBottomBorder;
		}

		if ( iDirection == 4 )
		{
			autoY = actualy-(autoHeight-height);
		}
		if ( iWrapDirection == 4 )
		{
			autoX = actualx-(autoWidth-width);
		}

		if ( actualx != autoX || actualy != autoY || actualwidth != autoWidth || actualheight != autoHeight )
		{
			Reposition(3, autoX, autoY, autoWidth, autoHeight, iAutoSizeSteps, iAutoSizeDelay);
		}
		else
		{
			pDontPaintAddedTrayIcon = NULL;
			pAddedTrayIcon = NULL;
		}
	}
}

bool Tray::IconAdd(NOTIFYICONDATA *pnid)
{
	if ( !IsWindow(pnid->hWnd) )
		return false;

	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit )
	{
		if ( (*tbit)->hWndOwner == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			return IconModify(pnid);
		}
	}

	TrayButton *button = new TrayButton( this, pnid->hWnd, pnid->uID );
	listTrayButtons.insert(listTrayButtons.end(), button);

	if ( (pnid->uFlags & NIF_MESSAGE) != 0 )
		button->uCallbackMessage = pnid->uCallbackMessage;

	if ( (pnid->uFlags & NIF_TIP) != 0 )
		button->strTooltip = (string) pnid->szTip;

	if ( (pnid->uFlags & NIF_ICON) != 0 )
		button->SetIcon( pnid->hIcon );

	if ( (pnid->uFlags & NIF_INFO) != 0 && IsOS( OS_2KXP ) )
	{
		if ( (int) pnid->uTimeout >= 0 )
			button->iTimeOut = min( max((int) pnid->uTimeout, INFOTIP_MINIMUM_TIMEOUT), INFOTIP_MAXIMUM_TIMEOUT);
		
		button->strInfo = (string) pnid->szInfo;
		button->strTitle = (string) pnid->szInfoTitle;

		button->dwInfoFlags = pnid->dwInfoFlags;

		if ( bInfoTipsEnabled && bShowToolTip && bShowInfoTips )
		{
			UpdateConfig.x = button->iButtonX+4;
			UpdateConfig.y = button->iButtonY+4;
			UpdateConfig.button = button;
			KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
			SetTimer(hWnd, TIMER_INFOTIPTIMEOUT, 1000, 0);
		}
	}

	if ( SortList.empty() && (bAutoSize || StrStrI(trayAdd.c_str(), "xtrayresize") != NULL || StrStrI(trayAdd.c_str(), "xtrayreposition") != NULL) )
		pDontPaintAddedTrayIcon = button;

	pAddedTrayIcon = button;

	return true;
}

bool Tray::IconModify(NOTIFYICONDATA *pnid)
{
	bool result = false;
	bool autoShowInfoTip = false;
	bool changeActiveTooltip = false;
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit )
	{	
		if ( (*tbit)->hWndOwner == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			if ( (*tbit)->Update(pnid, changeActiveTooltip, autoShowInfoTip) )
				result = true;

			if ( changeActiveTooltip && pTrayTooltip )
				pTrayTooltip->apply((*tbit)->strTooltip.c_str());

			if ( (!(*tbit)->bUserHidden || ((*tbit)->bUserHidden && bHiddenVisible)) && bInfoTipsEnabled && autoShowInfoTip && bShowToolTip && bShowInfoTips )
			{
				UpdateConfig.x = (*tbit)->iButtonX+4;
				UpdateConfig.y = (*tbit)->iButtonY+4;
				UpdateConfig.button = (*tbit);
				KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
				SetTimer(hWnd, TIMER_INFOTIPTIMEOUT, 1000, 0);
			}

			return result;
		}
	}	

	return IconAdd(pnid);
}

int Tray::IconDelete(NOTIFYICONDATA *pnid)
{
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit )
	{	
		if ( (*tbit)->hWndOwner == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			if ( UpdateConfig.button == (*tbit) )
				UpdateConfig.button = NULL;

			if ( pCapture == (*tbit) )
			{
				pCapture->bUseHover = false;
				pCapture = NULL;
			}

			int reallyHide = 1;

			if ( (*tbit)->bUserHidden && !bHiddenVisible )
				reallyHide = -1;
			else
				bRemovedTrayIcon = true;

			delete (*tbit);
			listTrayButtons.erase( tbit );
			
			return reallyHide;
		}
	}

	return 0;
}

bool Tray::IconCleanUp()
{
	for (TrayButtonListIterator tbit = listTrayButtons.begin(); tbit != listTrayButtons.end(); ++tbit )
	{	
		if ( !IsWindow((*tbit)->hWndOwner) )
		{
			if ( UpdateConfig.button == (*tbit) )
				UpdateConfig.button = NULL;

			if ( pCapture == (*tbit) )
			{
				pCapture->bUseHover = false;
				pCapture = NULL;
			}

			delete (*tbit);
			listTrayButtons.erase( tbit );
			
			return true;
		}
	}	
	
	return false;
}

bool Tray::CreateInfoTipWindow()
{
	bool br = false;

	hWndInfoTip = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
									TOOLTIPS_CLASS,
									TEXT(""),
									WS_POPUP | TTS_BALLOON | TTS_NOPREFIX | TTS_ALWAYSTIP,
									0, 0, 0, 0,
									0,
									0,
									hInstance,
									0);

	if ( hWndInfoTip )
	{
		SetWindowLong(hWndInfoTip, GWL_USERDATA, MAGIC_DWORD);

		SendMessage(hWndInfoTip, TTM_SETMAXTIPWIDTH, 0, INFOTIP_MAXIMUM_WIDTH);

		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)hWndInfoTip;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		SetRectEmpty(&ti.rect);

		// this fixes the mouse event bug on windows 2000
		SendMessage(hWndInfoTip, CCM_SETVERSION, COMCTL32_VERSION, 0);

		SendMessage(hWndInfoTip, (UINT) TTM_SETTIPBKCOLOR, (WPARAM) (COLORREF) clrInfoTipColor, 0 );   
		SendMessage(hWndInfoTip, (UINT) TTM_SETTIPTEXTCOLOR, (WPARAM) (COLORREF) clrInfoTipTextColor, 0);  

		// add the toolinfo to the window
		br = SendMessage(hWndInfoTip, TTM_ADDTOOL, 0, (LPARAM)(LPTOOLINFO)&ti);
	}
	
    return br;
}

void Tray::DestroyInfoTip()
{
	if (hWndInfoTip)
	{
	    TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)hWndInfoTip;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;
		
		SendMessage(hWndInfoTip, TTM_DELTOOL, 0, (LPARAM)(LPTOOLINFO)&ti);

		KillTimer(hWnd, INFOTIMER_AUTOHIDE);
	
		DestroyWindow(hWndInfoTip);
		hWndInfoTip = NULL;
	}
}

void Tray::ShowInfoTip(TrayButton *button, int x, int y)
{
	if ( hWndInfoTip )
	{
		RECT r;
		GetWindowRect(hWnd, &r);
		
		x = r.left+x;
		y = r.top+y;

		int icon = 0;
		// work out the icon
		if ( (button->dwInfoFlags & NIIF_INFO) != 0 )
			icon = 1;
		else if ( (button->dwInfoFlags & NIIF_WARNING) != 0 )
			icon = 2;
		else if ( (button->dwInfoFlags & NIIF_ERROR) != 0 )
			icon = 3;
		else if ( IsOS( OS_XP ) && (button->dwInfoFlags & NIIF_USER) != 0 )
			icon = (int) button->hIcon;
		else
			icon = 0;

		if (button->strTitle.empty())
			icon = 0;
	
		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)hWndInfoTip;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;

		string tooltip = "";
		if ( button->strInfo.empty() && button->strTitle.empty() )
			tooltip = button->strTooltip;
		else
			tooltip = button->strInfo;

		if ( button->bUserHidden )
			tooltip.append("\nCurrent State In xTray: Hidden");
			
		ti.lpszText = (char *) tooltip.c_str();

		ti.rect.left = r.left+button->iButtonX;
		ti.rect.right = ti.rect.left+button->iButtonWidth;
		ti.rect.top = r.top+button->iButtonY;
		ti.rect.bottom = ti.rect.top+button->iButtonHeight;

		SendMessage(hWndInfoTip, TTM_SETTOOLINFO, 0, (LPARAM)(LPTOOLINFO)&ti);
			
		if ( button->strTitle.empty() )
			SendMessage(hWndInfoTip, TTM_SETTITLE, (WPARAM) 0, (LPARAM) "\0");
		else
			SendMessage(hWndInfoTip, TTM_SETTITLE, (WPARAM) icon, (LPARAM) button->strTitle.c_str());

		SendMessage(hWndInfoTip, TTM_TRACKPOSITION, 0, (LPARAM) (DWORD) MAKELONG(x, y));
		SendMessage(hWndInfoTip, TTM_TRACKACTIVATE, TRUE, (LPARAM) (LPTOOLINFO) &ti);

		// set the timer
		KillTimer(hWnd, INFOTIMER_AUTOHIDE);
		SetTimer(hWnd, INFOTIMER_AUTOHIDE, button->iTimeOut, NULL);
	}
}

void Tray::HideInfoTip()
{
	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.uId = (UINT)hWndInfoTip;
	ti.hwnd = hWnd;
	ti.uFlags = TTF_TRACK | TTF_IDISHWND;

	SendMessage(hWndInfoTip, TTM_TRACKACTIVATE, FALSE, (LPARAM) (LPTOOLINFO) &ti);

	KillTimer(hWnd, INFOTIMER_AUTOHIDE);
}
