/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

    Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Utilities.h"

#define INVALID_ICON(x) ((NULL == (x)) || (1 == (int)(x)) || (-1 == (int)(x)))

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

string IntToString(int input)
{
	char buffer[32] = {0};
	itoa( input, buffer, 10 );
	return (string) buffer;
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five, string &six, string &seven)
{
	int count = 0;
	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	six = "";
	seven = "";
	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char csix[256] = {0};
	char cseven[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive, csix, cseven};
		
	int numTokens = LCTokenize( process.c_str(), cbuffers, 7, NULL);
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		
			if ( numTokens >= 3 )
			{
				three = cthree;
				count++;
			
				if ( numTokens >= 4 )
				{
					four = cfour;
					count++;

					if ( numTokens >= 5 )
					{
						five = cfive;
						count++;

						if ( numTokens >= 6 )
						{
							six = csix;
							count++;
		
							if ( numTokens >= 7 )
							{
								seven = cseven;
								count++;
							}
						}
					}
				}
			}
		}
	}

	return count;
}

string iGetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	GetxRCString(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetx2RCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	Getx2RCString(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	GetxRCLine(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetx2RCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	Getx2RCLine(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

string GetTaskClass( HWND hTask )
{
	char windowClass[MAX_LINE_LENGTH] = {0};

	if (hTask)
		GetClassName( hTask, windowClass, MAX_LINE_LENGTH );

	return (string) windowClass;
}

string GetTaskWindowText( HWND hTask )
{
	char windowText[MAX_LINE_LENGTH] = {0};

	if (hTask)
		GetWindowText( hTask, windowText, MAX_LINE_LENGTH );

	return (string) windowText;
}

bool IsSubString(const string &big, const string &small)
{
	if( match( small.c_str(), big.c_str() ) )
		return true;
	else
		return false;
}

bool IsOnTrayList(HWND hWnd, vector<CLASS_CONFIG> theList, bool emptyValue)
{	
	//Matching Lists
	if ( theList.empty() )
		return emptyValue;

	string tmpClass = GetTaskClass( hWnd );
	string tmpWindow = GetTaskWindowText( hWnd );

	if ( tmpClass.empty() )
		return emptyValue;
	
	for (vector<CLASS_CONFIG>::iterator it = theList.begin(); it != theList.end(); ++it)
	{
		if ( IsSubString(tmpClass, (*it).classname) )
		{
			if ( !(*it).windowname.empty() )
			{
				if ( !IsSubString(tmpWindow, (*it).windowname) )
					continue;
			}

			return !emptyValue;
		}
	}

	return emptyValue;
}

int PosOnTrayList(HWND hWnd, vector<CLASS_CONFIG> theList, int emptyValue)
{	
	//Matching Lists
	if ( theList.empty() )
		return emptyValue;

	string tmpClass = GetTaskClass( hWnd );
	string tmpWindow = GetTaskWindowText( hWnd );

	if ( tmpClass.empty() )
		return emptyValue;
	
	for (vector<CLASS_CONFIG>::iterator it = theList.begin(); it != theList.end(); ++it)
	{
		if ( IsSubString(tmpClass, (*it).classname) )
		{
			if ( !(*it).windowname.empty() )
			{
				if ( !IsSubString(tmpWindow, (*it).windowname) )
					continue;
			}

			return (*it).position;
		}
	}

	return emptyValue;
}

StringList GetxRCLineList(const string &uniqueKey)
{
	StringList resultList;
	LPVOID f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				resultList.insert(resultList.end(), configline);
			}
		}
		
		LCClose(f);
	}

	return resultList;
}

void GetxRCList(char *text, vector<CLASS_CONFIG> &theList, bool setPosition)
{
	LPVOID f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH] = {0};

		while(LCReadNextConfig(f, text, lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64] = {0};
			char first[MAX_LINE_LENGTH] = {0};
			char second[MAX_LINE_LENGTH] = {0};
			char third[MAX_LINE_LENGTH] = {0};
			char *buffers[] = {temp, first, second, third};

			CLASS_CONFIG hideCFG;
			
			int numTokens = LCTokenize(lineBuffer, buffers, 4, NULL);

			if ( !setPosition && numTokens == 2 )
			{
				hideCFG.classname = first;
				hideCFG.windowname = "";
				hideCFG.position = -1;

				if ( !hideCFG.classname.empty() )
					theList.insert(theList.end(), hideCFG);
			}
			else if ( !setPosition && numTokens == 3 )
			{
				hideCFG.classname = first;
				hideCFG.windowname = second;
				hideCFG.position = -1;

				if ( !hideCFG.classname.empty() )
					theList.insert(theList.end(), hideCFG);
			}
			else if ( setPosition && numTokens == 3 )
			{
				hideCFG.position = atoi(first);
				hideCFG.classname = second;
				hideCFG.windowname = "";
				
				if ( !hideCFG.classname.empty() )
					theList.insert(theList.end(), hideCFG);
			}
			else if ( setPosition && numTokens == 4 )
			{	
				hideCFG.position = atoi(first);
				hideCFG.classname = second;
				hideCFG.windowname = third;
			
				if ( !hideCFG.classname.empty() )
					theList.insert(theList.end(), hideCFG);
			}
		}
		
		LCClose(f);
	}
}

